////////////////////////////////////////////////////////////////////////////////
//
//		Licensed Materials - Property of IBM
//  		xxxx-xxx
//		(C) Copyright IBM Corp. 2004. All rights reserved.
//		US Government Users Restricted Rights - Use, duplication or disclosure
//		restricted by GSA ADP Schedule Contract with IBM Corp.
//
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has been
// deposited with the U.S. Copyright Office.
//
////////////////////////////////////////////////////////////////////////////////
package com.ibm.debug.idebug.engine;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.osgi.service.environment.Constants;

import com.ibm.debug.daemon.CoreDaemon;
import com.ibm.debug.pdt.IPDTDebuggerEngine;

/**
 * This class is used to discover and launch the PICL engine
 */
public class PiclEngineLauncher implements IPDTDebuggerEngine {
	protected static Vector	_engines	= new Vector();
	protected String		_engineName	= null;
	protected String		_enginePath	= null;

	public PiclEngineLauncher() {
		_engineName = (Platform.getOS().equals(Constants.OS_WIN32)) ? "irmtdbgc.exe"
			: "irmtdbgc";

		File engine = EnginePlugin.getDefault().findEngineFile(_engineName);
		try {
			if (engine != null)
				_enginePath = engine.getCanonicalPath();
		} catch (IOException e) {
			_enginePath = null;
		}
	}

	/**
	 * @return Iterates over all the currently running EngineProcess objects
	 */
	public Iterator getRunningEngines() {
		return (_engines == null) ? null : _engines.iterator();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.debug.pdt.IPDTDebuggerEngine#getLocation()
	 */
	public URL getInstallDirectory() {
		if (_enginePath != null && Path.EMPTY.isValidPath(_enginePath)) {
			IPath installRoot = new Path(_enginePath);
			
			if (installRoot.lastSegment().equals(_engineName))
				installRoot = installRoot.removeLastSegments(1);

			if (installRoot.lastSegment().toLowerCase().equals("bin"))
				installRoot = installRoot.removeLastSegments(1);

			try {
				return new File(installRoot.toOSString()).toURI().toURL();
			} catch (MalformedURLException e) {
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.debug.pdt.IPDTDebuggerEngine#launchEngine()
	 */
	public boolean launchEngine(int key, String[] environment) {
		if (_enginePath == null)
			return false;

		String command = _enginePath;

		command += " -qhost=localhost -quiport="
			+ (new Integer(CoreDaemon.getCurrentPort())).toString();

		if (key >= 1) {
			command += " -startupKey=" + (new Integer(key)).toString();
		}

		try {
			EngineProcess process = new EngineProcess(Runtime.getRuntime()
				.exec(command, environment));
			_engines.add(process);
		} catch (SecurityException e) {
			return false;
		} catch (IOException e) {
			return false;
		}

		return true;
	}

	private static final String	IBMCopyRight	= "(C) Copyright IBM Corp. 2004. All rights reserved.";
}